@@ -5,6 +5,7 @@ from .validation import (
     DataSource,
     RequestSchemaValidationError,
     ResponseSchemaValidationError,
+    validate_headers,
     validate_querystring,
     validate_request,
     validate_response,
@@ -19,6 +20,7 @@ __all__ = (
     "ResponseSchemaValidationError",
     "SchemaValidationError",
     "tag",
+    "validate_headers",
     "validate_querystring",
     "validate_request",
     "validate_response",
