@@ -3,7 +3,7 @@ from __future__ import annotations
 from dataclasses import asdict, is_dataclass
 from enum import auto, Enum
 from functools import wraps
-from typing import Any, Callable, cast, Optional, Union
+from typing import Any, Callable, cast, Dict, Optional, Tuple, Type, TypeVar, Union
 
 from pydantic import BaseModel, ValidationError
 from pydantic.dataclasses import dataclass as pydantic_dataclass, is_builtin_dataclass
@@ -14,6 +14,7 @@ from werkzeug.exceptions import BadRequest
 
 from .typing import PydanticModel, ResponseReturnValue
 
+QUART_SCHEMA_HEADERS_ATTRIBUTE = "_quart_schema_headers_schema"
 QUART_SCHEMA_REQUEST_ATTRIBUTE = "_quart_schema_request_schema"
 QUART_SCHEMA_RESPONSE_ATTRIBUTE = "_quart_schema_response_schemas"
 QUART_SCHEMA_QUERYSTRING_ATTRIBUTE = "_quart_schema_querystring_schema"
@@ -28,12 +29,24 @@ class ResponseSchemaValidationError(Exception):
         self.validation_error = validation_error
 
 
+class ResponseHeadersValidationError(ResponseSchemaValidationError):
+    pass
+
+
 class RequestSchemaValidationError(BadRequest):
     def __init__(self, validation_error: Union[TypeError, ValidationError]) -> None:
         super().__init__()
         self.validation_error = validation_error
 
 
+class QuerystringValidationError(RequestSchemaValidationError):
+    pass
+
+
+class RequestHeadersValidationError(RequestSchemaValidationError):
+    pass
+
+
 class DataSource(Enum):
     FORM = auto()
     JSON = auto()
@@ -67,7 +80,7 @@ def validate_querystring(model_class: PydanticModel) -> Callable:
             try:
                 model = model_class(**request.args)
             except (TypeError, ValidationError) as error:
-                raise RequestSchemaValidationError(error)
+                raise QuerystringValidationError(error)
             else:
                 return await current_app.ensure_async(func)(*args, query_args=model, **kwargs)
 
@@ -76,6 +89,39 @@ def validate_querystring(model_class: PydanticModel) -> Callable:
     return decorator
 
 
+def validate_headers(model_class: PydanticModel) -> Callable:
+    """Validate the headers.
+
+    This ensures that the headers can be converted to the
+    *model_class*. If they cannot a `RequestSchemaValidationError` is
+    raised which by default results in a 400 response.
+
+    Arguments:
+        model_class: The model to use, either a dataclass, pydantic
+            dataclass or a class that inherits from pydantic's
+            BaseModel.
+
+    """
+    if is_builtin_dataclass(model_class):
+        model_class = pydantic_dataclass(model_class)
+
+    def decorator(func: Callable) -> Callable:
+        setattr(func, QUART_SCHEMA_HEADERS_ATTRIBUTE, model_class)
+
+        @wraps(func)
+        async def wrapper(*args: Any, **kwargs: Any) -> Any:
+            try:
+                model = _convert_headers(request.headers, model_class)
+            except (TypeError, ValidationError) as error:
+                raise RequestHeadersValidationError(error)
+            else:
+                return await current_app.ensure_async(func)(*args, headers=model, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
 def validate_request(
     model_class: PydanticModel,
     *,
@@ -126,7 +172,11 @@ def validate_request(
     return decorator
 
 
-def validate_response(model_class: PydanticModel, status_code: int = 200) -> Callable:
+def validate_response(
+    model_class: PydanticModel,
+    status_code: int = 200,
+    headers_model_class: Optional[PydanticModel] = None,
+) -> Callable:
     """Validate the response data.
 
     This ensures that the response is a either dictionary that the
@@ -139,18 +189,24 @@ def validate_response(model_class: PydanticModel, status_code: int = 200) -> Cal
     Arguments:
         model_class: The model to use, either a dataclass, pydantic
             dataclass or a class that inherits from pydantic's
-            BaseModel. All the fields must be optional.
+            BaseModel.
         status_code: The status code this validation applies
             to. Defaults to 200.
+         headers_model_class: The model to use to validate response
+            headers, either a dataclass, pydantic dataclass or a class
+            that inherits from pydantic's BaseModel. Is optional.
     """
     if is_builtin_dataclass(model_class):
         model_class = pydantic_dataclass(model_class)
 
+    if is_builtin_dataclass(headers_model_class):
+        headers_model_class = pydantic_dataclass(headers_model_class)
+
     def decorator(
         func: Callable[..., ResponseReturnValue]
     ) -> Callable[..., QuartResponseReturnValue]:
         schemas = getattr(func, QUART_SCHEMA_RESPONSE_ATTRIBUTE, {})
-        schemas[status_code] = model_class
+        schemas[status_code] = (model_class, headers_model_class)
         setattr(func, QUART_SCHEMA_RESPONSE_ATTRIBUTE, schemas)
 
         @wraps(func)
@@ -165,31 +221,95 @@ def validate_response(model_class: PydanticModel, status_code: int = 200) -> Cal
                 value = result
 
             status = 200
-            if status_or_headers is not None and not isinstance(
-                status_or_headers, (Headers, dict, list)
-            ):
+            if isinstance(status_or_headers, int):
                 status = int(status_or_headers)
 
             if status == status_code:
-                if isinstance(value, dict):
-                    try:
+                try:
+                    if isinstance(value, dict):
                         model_value = model_class(**value)
-                    except ValidationError as error:
-                        raise ResponseSchemaValidationError(error)
-                elif type(value) == model_class:
-                    model_value = value
-                elif is_builtin_dataclass(value):
-                    model_value = model_class(**asdict(value))
-                else:
-                    raise ResponseSchemaValidationError()
+                    elif type(value) == model_class:
+                        model_value = value
+                    elif is_builtin_dataclass(value):
+                        model_value = model_class(**asdict(value))
+                    else:
+                        raise ResponseSchemaValidationError()
+                except ValidationError as error:
+                    raise ResponseSchemaValidationError(error)
+
                 if is_dataclass(model_value):
-                    return asdict(model_value), status_or_headers, headers
+                    return_value = asdict(model_value)
                 else:
-                    model_value = cast(BaseModel, model_value)
-                    return model_value.dict(), status_or_headers, headers
+                    return_value = cast(BaseModel, model_value).dict()
+
+                if headers_model_class is not None:
+                    try:
+                        if isinstance(headers, dict):
+                            headers_model_value = _convert_headers(headers, headers_model_class)
+                        elif type(value) == headers_model_class:
+                            headers_model_value = headers
+                        elif is_builtin_dataclass(headers):
+                            headers_model_value = headers_model_class(**asdict(headers))
+                        else:
+                            raise ResponseHeadersValidationError()
+                    except ValidationError as error:
+                        raise ResponseHeadersValidationError(error)
+
+                    if is_dataclass(headers_model_value):
+                        headers_value = asdict(headers_model_value)
+                    else:
+                        headers_value = cast(BaseModel, headers_model_value).dict()
+                else:
+                    headers_value = headers
+
+                return return_value, status, headers_value
             else:
                 return result
 
         return wrapper
 
     return decorator
+
+
+def validate(
+    *,
+    querystring: Optional[PydanticModel] = None,
+    request: Optional[PydanticModel] = None,
+    request_source: DataSource = DataSource.JSON,
+    headers: Optional[PydanticModel] = None,
+    responses: Dict[int, Tuple[PydanticModel, Optional[PydanticModel]]],
+) -> Callable:
+    """Validate the route.
+
+    This is a shorthand combination of of the validate_querystring,
+    validate_request, validate_headers, and validate_response
+    decorators. Please see the docstrings for those decorators.
+    """
+
+    def decorator(func: Callable) -> Callable:
+        if querystring is not None:
+            func = validate_querystring(querystring)(func)
+        if request is not None:
+            func = validate_request(request, source=request_source)(func)
+        if headers is not None:
+            func = validate_headers(headers)(func)
+        for status, models in responses.items():
+            func = validate_response(models[0], status, models[1])
+        return func
+
+    return decorator
+
+
+T = TypeVar("T")
+
+
+def _convert_headers(headers: Union[dict, Headers], model_class: Type[T]) -> T:
+    result = {}
+    for raw_key in headers.keys():
+        key = raw_key.replace("-", "_").lower()
+        if key in model_class.__annotations__:
+            if isinstance(headers, Headers):
+                result[key] = ",".join(headers.get_all(raw_key))
+            else:
+                result[key] = headers[raw_key]
+    return model_class(**result)
